/*M*                     
//                    INTEL CORPORATION PROPRIETARY INFORMATION
//      This software is supplied under the terms of a license agreement or
//      nondisclosure agreement with Intel Corporation and may not be copied
//      or disclosed except in accordance with the terms of that agreement.
//            Copyright (c) 1995 Intel Corporation.  All Rights Reserved.
//
*M*/
/*
 * $Log:   //epg-scc/sst/ProjectDatabases/PI5x/archives/PI 5.0 NetWare/work/SRC/SITKA/IMB_NW.C.v  $
 * 
 *    Rev 1.13   28 Aug 2002 16:38:34   dmondal
 * Modified for IPMI interface detection
 * 
 *    Rev 1.12   16 Jul 2001 17:29:24   admatkax
 * Tracker bug 15459
 *
 *    Rev 1.11   26 Jun 2001 16:39:42   admatkax
 * Get message & send message are changed accordingly to incorporate the changes introduced in IPMI1.5
 * 
 *    Rev 1.10   11 Jun 2001 17:06:42   spoola
 * Fixed missing break statement in switch case in getbmcipmiversion function.
 * 
 *    Rev 1.9   25 Apr 2001 09:48:32   svuppula
 * Added GetIpmiVersion
 * 
 *    Rev 1.8   Apr 19 2001 18:15:06   ctran2
 * Implementation of GetIpmiVersion() function (similar to
 * GetIpmiVersion() function in imbapi\imbapi.c)
 * 
 *    Rev 1.7   Sep 21 2000 17:33:44   simont
 * Fix for 23736.  The Async message thread
 * needs to be synchronized with the other
 * Imb messages.
 * 
 * 
 *    Rev 1.6   Jul 11 2000 21:20:08   simont
 * Fix for 12649.  GetMessage for EMP was
 * incorrectly picking up message over LAN.
 * 
 *    Rev 1.5   May 26 2000 17:03:16   simont
 * Recheck in 3.0 Changes.
 * 
 *    Rev 1.3   May 17 2000 09:50:26   simont
 * lun number needs to be 0 even for DPC
 * over Lan.  The firmware spec is currently
 * not correct.
 * 
 *    Rev 1.2   01 May 2000 18:00:26   simont
 * Add DPC over Lan support.
 * 
 *    Rev 1.1   09 Feb 2000 11:52:08   klperkin
 * PI 3.0 BRANCH TIP
 * 
 *    Rev 1.13   29 Dec 1999 14:49:10   jpaiasno
 * added 'DriveReady' flag to block all 
 * app.  calls till the driver is intialized
 * and IPMI version is properly detected.
 * 
 *    Rev 1.12   28 Dec 1999 11:18:50   jpaiasno
 * Refined logic related to detection of IPMI version
 * 
 *    Rev 1.11   15 Nov 1999 17:18:26   jpaiasno
 * Fixed logic related to determination of
 * the IPMI version supported by BMC 
 * 
 *    Rev 1.10   18 Aug 1999 18:10:14   sganeshx
 * fixed DPC over NetWare bug.
 * 1. Modified function 'UnRegisterForImbAsyncMessageNotification'
 *     in file IMB_NW.C to accept a second parameter
 *     which will indicate source of NLM unload.
 * 2. Modified function prototype in IMBAPI.H
 * 3. Modified call to function 'UnRegisterForImbAsyncMessageNotification'
 *     in DPCTHD.C
 * 
 *    Rev 1.9   27 Jul 1999 17:17:32   jpaiasno
 * Changed BMC communication maximum buffer size to 64
 * bytes to support all IPMI 1.0 based commands.
 * 
 * 
 *    Rev 1.8   Jul 09 1999 17:42:44   sganeshx
 * 1. added fixes to address IPMI v1.0 changes
 *     to function 'SendTimedEmpMessageResponse'
 * 
 * 
 *    Rev 1.7   24 May 1999 15:28:06   jpaiasno
 * Added code to properly handle the
 * unregistration for DPC related Async messages.
 * 
 *    Rev 1.6   May 14 1999 09:39:34   sganeshx
 * 1. ported changes in function 
 *     'GetAsyncImbpMessage' from IMB source
 *    for NT
 * 2. removed unreferenced variables (eliminates
 *     all compile time warnings about unreferenced
 *     variables)
 * 
 *    Rev 1.5   May 07 1999 14:10:40   sganeshx
 * 1. response length from 'Get Device ID' command
 *     is now compared against (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1)
 *    for IPMI v1.0
 * 
 *    Rev 1.4   Apr 09 1999 17:28:16   sganeshx
 * Added check to see if semaphore handles
 * are valid before closing them. This used to
 * cause an abend in NetWare 4.2 and 
 * generate warnings in NetWare 5.0.
 * 
 *    Rev 1.3   Mar 31 1999 15:09:00   sganeshx
 * fixed bug that caused IMBDRV to timeout on
 * startup (in function getBmcIpmiVer)
 * 
 *    Rev 1.2   21 Jan 1999 14:31:02   jpaiasno
 * Added code to support IPMI 1.0 firmware
 * 
 *    Rev 1.1   10 Dec 1998 13:41:46   jpaiasno
 * added code to supprt Async DPC events
 * 
 *    Rev 1.0   Sep 20 1998 18:31:28   whcernan
 * Initial revision.
 * 
 *    Rev 1.19   02 Jun 1998 10:00:14   hfcheng
 * Bug fix for 5263.  A check is put in the 
 * SendTimedI2cRequest() and 
 * SendTimeImbpRequest() to make sure that
 * the response pointer is not NULL before
 * doing a memcpy().
 * 
 * 
 *    Rev 1.18   26 May 1998 17:51:12   hfcheng
 * continue to fix for 5034.  In the previous
 * check in, I miss the change in the module
 * GetAsyncImbpMessage()
 * 
 *    Rev 1.17   26 May 1998 15:22:06   jpaiasno
 * Added code to set the Imbdriver's  thread group Id when
 * the NLM is called in 'GetAsyncImbpMessage'
 * and other APIs.
 * 
 * 
 *    Rev 1.15.1.0   22 May 1998 11:36:20   jpaiasno
 * porting from 1.16 in the main branch to fix the unload
 * problem
 * 
 *    Rev 1.16   22 May 1998 11:12:58   jpaiasno
 * Fixed the NLM  unload problem by freeing
 * the allocated que memory and bypassing the
 * semaphor wait in the check function
 * 
 *    Rev 1.15   14 May 1998 15:57:22   hfcheng
 * fix for 5034.  SendTimedRequest() and 
 * SendAsyncImbpRequest() should return
 * ACCESN_ERROR only after 
 * SendImbRequest() returns non-zero status.
 * The original code will return ACCESN_ERROR
 * if the status is zero (IMB_SUCCESS) which
 * is wrong.
 * 
 *    Rev 1.14   15 Apr 1998 14:59:46   fkhanx
 * Added more comments describing the semaphore
 * pool mechanism
 * 
 *    Rev 1.13   15 Apr 1998 13:56:08   fkhanx
 * Added the semaphore pool cleanup code
 * 
 *    Rev 1.12   15 Apr 1998 12:21:54   fkhanx
 * Fixed compilation errors
 * 
 *    Rev 1.11   14 Apr 1998 21:39:38   fkhanx
 * Added code for the Async Message portion 
 * of the driver.
 * 
 *    Rev 1.10   Apr 11 1998 11:12:30   Administrator
 * Added dummy Async message  API calls
 * for EIF debugging .
 * 
 *    Rev 1.9   07 Apr 1998 00:31:12   fkhanx
 * Fixed the retry count problem.Now the 
 * Imb Driver runs at full speed.
 * 
 *    Rev 1.8   24 Mar 1998 16:00:18   fkhanx
 * Added copyright headers
*/
#include "imb_os.h"
#include "imb_if.h"
#include "imbapi.h"
#include "imb_asyn.h"
#include "imb_drv.h"
#include "imb_log.h"
#include "SMBIOS.H"
#undef NULL
#include <stdio.h> 
#include <stdlib.h> 
#include <process.h>
#include <nwsemaph.h>
#include <advanced.h>
#include <conio.h>
#include <errno.h>
#include <malloc.h>
#include <string.h>


#define ERR_TIMEOUT_FAILURE 0xfe
#define S_NO_ACTION                           0
#define BMC_CONTROLLER 0x20
#define NET_FUNCTION_APPLICATION 6
#define S_RESET 1
#define S_POWER_OFF 2

// extern function prototypes
extern LONG MapAbsoluteAddressToDataOffset(unsigned char *);

//flags to handle unregistration for Async Events

static int AsyncEventUnregistrationFlag = 0;
static int AckAsyncEventUnregistrationFlag = 0;
static unsigned char TimeOutFlag = 0;
static int DriverReady = 0;
unsigned char PoolDisabled=0;
typedef struct _SemaphorePool
{
	unsigned PoolCount;
	unsigned char *BusyFlags;
	long *PoolMembers;
	long LockPool;
}SemaphorePool;

SemaphorePool semaPool;

#define INITIAL_POOL_SIZE 4

// synchronising semaphore  forrequest processing and timeout threads.
// synchronising semaphore for mismic hardware access by BMC
// and IMBP request processing functions. This semaphore allows
// reentrancy in the driver for mismic access. 
static LONG      TimeOutSemphr, MismicAccessSemphr ;
static LONG               AsyncEventHandle = 0;


//timeout value provided by the calling agent
static int       TimeOutValue;


//Thread and group Ids
static int       TimeOutThreadId = -1;
static int       MainThreadId;
static int       GlobalThreadGroupId;

// variables used for gracefull  nlm unload
static char      SuspendActivityFlag = 0, threadKilled = 0;
static char      TimeOutThreadReady = 1, ResourceFree = 1;



static int    DelayTime = 1;
static int    ActionCode = S_NO_ACTION   ;
static long EventHandle;

unsigned char imbSeqNo=0;
ImbInterface * imbIf;
static int   ApiMapVirtualAddress = 0;
static unsigned char  BmcIpmiVersion = IPMI_OLD_VER;

ImbAsyncQueueInfo asyncQ;

#define MAX_MAPPED_ADDR 10
static int aApiMapVirtualAddress[MAX_MAPPED_ADDR];

/////////////////////////////////////////////////////////////////////////////////////////////////
// main 
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       main 
//  Purpose:    This is the main function of the mismic driver that creates the
//                              timeout thread and other synchronization objects and initializes the 
//                              Mismic and critical log migration interface.It also initiates periodic
//                              log migration.
//
//  Context:    called by Operating system during during this NLM load. 
//
//  Returns:    none
//  Parameters: none
//  Notes:      none
*F*/


main ()
{

	ImbInterface **		paimbIf;
	ImbInterface *		pimbIf = NULL;
	extern ImbInterface     * ImbInterfaces[];

	// Check to see if we have an interface that we recognize
	if(!GetIPMIDeviceInformation(&pimbIf))
	{
		ConsolePrintf ("Interface Detection from SMBIOS is failed \n");
		for( paimbIf = ImbInterfaces ; *paimbIf != NULL ; paimbIf++ ) 
		{
			if( (*paimbIf)->ifProbe() == TRUE )
				break;
		}
		
		imbIf = *paimbIf;		
	}
	else
	{
		imbIf = pimbIf;
		ConsolePrintf ("Interface Detection from SMBIOS is success \n");
	}

	if (!imbIf)
	{
		return;
	}

	imbIf->InitRegs ((void *)imbIf);
	imbSeqNo = 0;

	// moved here because this semaphore is accessed in getBmcIpmiVersion
    //create timeout thread synchronization semaphore
	TimeOutSemphr = OpenLocalSemaphore (0);

	GlobalThreadGroupId = GetThreadGroupID();

	MainThreadId = GetThreadID();

	EventHandle = RegisterForEvent(EVENT_EXIT_TO_DOS,
				   AtOSShutdown, NULL);

	// initialize the async messaging subsystem
	// it would be nice to get the queue size from the registry
	//
    asyncQ.queue = (ImbAsyncEntry *) malloc(IMB_ASYNC_QUEUE_SIZE * sizeof( ImbAsyncEntry ));
    asyncQ.queueSize    = IMB_ASYNC_QUEUE_SIZE;
	ImbAsyncInit(&asyncQ);

    //create Mismic hardware access synchronization semaphore
    MismicAccessSemphr =OpenLocalSemaphore(1);

    //create timeout generation thread
    TimeOutThreadId = BeginThread (TimeOutThread, 0, 8192, 0);
	//get the ipmi version of the BMC
	getBmcIpmiVersion();
	DriverReady =1;

	if ( BmcIpmiVersion == IPMI_UNKNOWN_VER)
	{
		// terminate the NLM
		SuspendActivityFlag = 1;
	}

    while ( 1 ) 
	{
      
		if (SuspendActivityFlag)
			break;
		// start looking for Async messages
	    WaitOnLocalSemaphore (MismicAccessSemphr);  
    	ImbAsyncMessageProcess(NULL); 
		SignalLocalSemaphore (MismicAccessSemphr);
		delay ( MISMIC_PROC_CALL_INTERVAL );          
      
    } 

	
	UnregisterForEvent(EventHandle);

    while(1)
	{

		if ((SuspendActivityFlag) && (threadKilled) && (ResourceFree) )  
		{

			// check if we have valid semaphore handles before we close them
			if (TimeOutSemphr != NULL)
			{
				CloseLocalSemaphore (TimeOutSemphr); 
				TimeOutSemphr = 0;
			}
			if (MismicAccessSemphr != NULL)
			{
				CloseLocalSemaphore(MismicAccessSemphr);
				MismicAccessSemphr = 0;
			}
			//PerformCleanup();
			UnregisterForEvent(EventHandle);

			//free the allocated memory and close the semaphores
			//(pai 5/21)
			if (asyncQ.queue != NULL)
				free(asyncQ.queue);

			
			if(asyncQ.syncSem != NULL)
			{

				CloseLocalSemaphore(asyncQ.syncSem);
				asyncQ.syncSem = 0;
			}
			if( semaPool.LockPool != NULL)
			{
				CloseLocalSemaphore(semaPool.LockPool);
				semaPool.LockPool = 0;
			}



		    SuspendActivityFlag = 2;
		    break;
		}
		else
		{

			SignalLocalSemaphore (TimeOutSemphr);
			delay(100);
		}
    }
  
    while(1)
	{
		delay(100);  
    }
}




/////////////////////////////////////////////////////////////////////////////////////////////////
// TimeOutThread 
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       TimeOutThread 
//  Purpose:    This is the timeout generation thread created by the main function
//
//  Context:    It is triggred by BMC and I2c access related functions, to generate 
//              time out during IMBP based frames send/receive. 
//
//  Returns:    none
//  Parameters: ptr   dummy
//  Notes:      none
*F*/

void TimeOutThread(void *ptr)                                     
{
    while(1) {       
	WaitOnLocalSemaphore (TimeOutSemphr);
	TimeOutThreadReady = 0;
	delay(TimeOutValue);
	if (TimeOutFlag)
	TimeOutFlag = 0;

	if ( SuspendActivityFlag) {
	    threadKilled = 1;
	    ExitThread(TSR_THREAD,0);
	} 

	TimeOutThreadReady = 1;
    }       
}

	typedef struct _WriteReadI2C {                          // format of a write/read I2C request
		BYTE    busType;
		BYTE    rsSa;
		BYTE    count;
		BYTE    data[1];
	} WriteReadI2C;

////////////////////////////////////////////////////////////////////////////////
// SendTimedI2cRequest 
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedI2cRequest 
//  Purpose:    This function sends a request to I2C Device
//  Context:    Used by Upper level agents access dumn I2c devices. 
//  Returns:    ACCES_OK  else error status code
//  Parameters: 
//          reqPtr  
//          timeOut  
//          respDataPtr
//          respLen 
//  Notes:      none
*F*/
ACCESN_STATUS
SendTimedI2cRequest (
		I2CREQUESTDATA *        reqPtr,                 // I2C request
		int                                     timeOut,                // how long to wait, mSec units
		BYTE *                          respDataPtr,    // where to put response data
		int *                           respDataLen,    // size of response buffer/size of returned data
		BYTE *                          completionCode  // request status from BMC
	)
{
	BOOL                            status;

    BYTE                                responseData[MAX_IMB_RESP_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int oldId;
	WriteReadI2C * wrReq;

	    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }

    while(!DriverReady)
     delay(400);


	wrReq = (WriteReadI2C *) req->req.data;

#define MIN_WRI2C_SIZE  3       // size of write/read request minus any data

	req->req.rsSa           = BMC_SA;
	req->req.cmd            = WRITE_READ_I2C;
	req->req.netFn          = APP_NETFN;
	req->req.rsLun          = BMC_LUN;
	req->req.dataLength     = reqPtr->dataLength + MIN_WRI2C_SIZE;

	wrReq->busType          = reqPtr->busType;
	wrReq->rsSa                     = reqPtr->rsSa;
	wrReq->count            = reqPtr->numberOfBytesToRead;
	
	memcpy( wrReq->data, reqPtr->data, reqPtr->dataLength );

	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units

    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);
	if( status ) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}
	if( respLength == 0 ) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}
    ResourceFree = 1;
    
	//
	// give the caller his response
	//
	*completionCode = resp->cCode;
	*respDataLen    = respLength - 1;

	if(( *respDataLen ) && ( respDataPtr))
		memcpy( respDataPtr, resp->data, *respDataLen);
	SetThreadGroupID(oldId);  

	return ACCESN_OK;
}

////////////////////////////////////////////////////////////////////////////////
// SendTimedImbpRequest 
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedImbpRequest 
//  Purpose:    This function sends a request for BMC implemented function
//  Context:    Used by Upper level agents to access BMC implemented
//              functionality. 
//  Returns:    ACCESN_OK  else error status code
//  Parameters: 
//    reqPtr       pointer to IMBP  based request data structure
//    timeOut      time out value in milliseconds
//    respDataPtr  pointer to the response data buffer
//    respLen      pointer to the response length. Higher level agent will pass
//                 this as a pointer to allocated buffer length. During return 
//                 it will point to the actual number of response bytes returned
//                 by the driver.
//  Notes:      none
*F*/
ACCESN_STATUS
SendTimedImbpRequest (
		IMBPREQUESTDATA *reqPtr,                // request info and data
		int                             timeOut,                // how long to wait, in mSec units
		BYTE *                  respDataPtr,    // where to put response data
		int *                   respDataLen,    // how much response data there is
		BYTE *                  completionCode  // request status from dest controller
	)
{
	BOOL                            status;

    BYTE                                responseData[MAX_BUFFER_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_BUFFER_SIZE     ];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int oldId;
	//
	// If the Imb driver is not present return AccessFailed
	//
    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }


    while(!DriverReady)
     delay(400);

      
	req->req.rsSa           = reqPtr->rsSa;
	req->req.cmd            = reqPtr->cmdType;
	req->req.netFn          = reqPtr->netFn;
	req->req.rsLun          = reqPtr->rsLun;
	req->req.dataLength     = reqPtr->dataLength;

	memcpy( req->req.data, reqPtr->data, reqPtr->dataLength );

	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units

    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);

	if( status) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}
	if( respLength == 0 ) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}

    ResourceFree = 1;
    //SetThreadGroupID(oldId);
	//
	// give the caller is response
	//
	*completionCode = resp->cCode;
	*respDataLen    = respLength - 1;

	if(( *respDataLen )  && ( respDataPtr))
		memcpy( respDataPtr, resp->data, *respDataLen);

	SetThreadGroupID(oldId);  

	return ACCESN_OK;
}







/////////////////////////////////////////////////////////////////////////////////////////////////
// SendTimedEmpMessageResponse
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedEmpMessageResponse 
//  Purpose:    This function sends a response message to the EMP port
//  Context:     
//  Returns:    OK  else error status code
//  Parameters: 
//  
//  Notes:      none
*F*/




ACCESN_STATUS
SendTimedEmpMessageResponse (

		ImbPacket *      ptr,           // pointer to the original request from EMP
		char      *responseDataBuf,
		int               responseDataLen,
		int             timeOut         // how long to wait, in mSec units
	)
{
	BOOL                            status;


    BYTE                                responseData[MAX_IMB_RESP_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int i,j;
	int oldId;

	//
	// If the Imb driver is not present return AccessFailed
	//
    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }



    while(!DriverReady)
     delay(400);




	//form the response packet first
	

	req->req.rsSa           =  BMC_SA;
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.cmd            =  WRITE_EMP_BUFFER;
	else 
		req->req.cmd            =  SEND_MESSAGE;        

	req->req.netFn          =  APP_NETFN;
	req->req.rsLun          =  0;

	i = 0;
	if (BmcIpmiVersion !=   IPMI_09_VERSION)
		req->req.data[i++]      = EMP_CHANNEL;

	req->req.data[i++]    =  ptr->rqSa;
	req->req.data[i++]      =  (((ptr->nfLn & 0xfc) | 0x4) | ((ptr->seqLn) & 0x3));
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.data[i++]    = ((~(req->req.data[0] +  req->req.data[1])) +1);
	else
		req->req.data[i++]    = ((~(req->req.data[1] +  req->req.data[2])) +1);
	req->req.data[i++]      =  BMC_SA; //though software is responding, we have to
								   //provide BMC's slave address as responder
								   //address.
	
	req->req.data[i++]      = ( (ptr->seqLn & 0xfc) | (ptr->nfLn & 0x3) );

	req->req.data[i++]      = ptr->cmd;
	for ( j = 0 ; j < responseDataLen ; ++j,++i)
	   req->req.data[i] = responseDataBuf[j];

	 req->req.data[i] = 0;
	 if (BmcIpmiVersion ==  IPMI_09_VERSION)
		 j = 0;
	 else 
		 j = 1;

	for (; j < ( i -3); ++j)
		 req->req.data[i] += req->req.data[j+3];
	req->req.data[i]  = ~(req->req.data[i]) +1;
	++i;
	req->req.dataLength     = i;

	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units




    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);

	if( status) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}







	

	if ( 
		(respLength != 1) || (responseData[0] != 0)
					
		)

		{

		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;

		}



		
	
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}



/////////////////////////////////////////////////////////////////////////////////////////////////
// SendTimedEmpMessageResponse_Ex
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedEmpMessageResponse_Ex 
//  Purpose:    This function sends a response message to the EMP port
//  Context:     
//  Returns:    OK  else error status code
//  Parameters: 
//  
//  Notes:      none
*F*/




ACCESN_STATUS
SendTimedEmpMessageResponse_Ex (

		ImbPacket *      ptr,           // pointer to the original request from EMP
		char      *responseDataBuf,
		int               responseDataLen,
		int             timeOut  ,       // how long to wait, in mSec units
		BYTE		sessionHandle,	//This is introduced in IPMI1.5,this is required to be sent in 
									//send message command as a parameter,which is then used by BMC
									//to identify the correct DPC session to send the mesage to.
		BYTE		channelNumber	//There are 3 different channels on which DPC communication goes on
									//Emp - 1,Lan channel one - 6,Lan channel two(primary channel) - 7.
	)
{
	BOOL                            status;


    BYTE                                responseData[MAX_IMB_RESP_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int i,j;
	int oldId;

	//
	// If the Imb driver is not present return AccessFailed
	//
    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }



    while(!DriverReady)
     delay(400);




	//form the response packet first
	

	req->req.rsSa           =  BMC_SA;
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.cmd            =  WRITE_EMP_BUFFER;
	else 
		req->req.cmd            =  SEND_MESSAGE;        

	req->req.netFn          =  APP_NETFN;
	req->req.rsLun          =  0;

	i = 0;

	//checking for the IPMI version & then assigning the channel number for EMP
	//Actually the channel number is same in both the versions.This is just to 
	//maintain the consistancy with the same method for LAN.
	//This is the 1st byte of the SEND MESSAGE command.
	if (BmcIpmiVersion ==	IPMI_10_VERSION)
		req->req.data[i++] 	= EMP_CHANNEL;
	else if (BmcIpmiVersion ==	IPMI_15_VERSION)
		req->req.data[i++] 	= channelNumber;

	//The second byte of data for SEND MESSAGE starts with session handle
	req->req.data[i++] = sessionHandle;

	//Then it is the response slave address for SEND MESSAGE.
	req->req.data[i++]    =  ptr->rqSa;

	//Then the net function + lun for SEND MESSAGE command.
	req->req.data[i++]      =  (((ptr->nfLn & 0xfc) | 0x4) | ((ptr->seqLn) & 0x3));

	//Here the checksum is calculated.The checksum calculation starts after the channel number.
	//so for the IPMI 1.5 version its a checksum of 3 bytes that is session handle,response slave 
	//address & netfun+lun.
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.data[i++]    = ((~(req->req.data[0] +  req->req.data[1])) +1);
	else
	{
		if (BmcIpmiVersion == IPMI_10_VERSION)
			req->req.data[i++]    = ((~(req->req.data[1] +  req->req.data[2])) +1);
        else
			req->req.data[i++]    = ((~(req->req.data[2]+  req->req.data[3])) +1);
	}

	//This is the next byte of the message data for SEND MESSAGE command.It is the request 
	//slave address.
	req->req.data[i++]      =  BMC_SA; //though software is responding, we have to
								   //provide BMC's slave address as responder
								   //address.
	
	//This is just the sequence number,which is the next byte of data for SEND MESSAGE
	req->req.data[i++]      = ( (ptr->seqLn & 0xfc) | (ptr->nfLn & 0x3) );

	//The next byte is the command like get software ID(00).
	req->req.data[i++]      = ptr->cmd;

	//after the cmd the data ,which is sent by DPC & is retrived using the get message earlier
	// is sent back to DPC.
	for ( j = 0 ; j < responseDataLen ; ++j,++i)
	   req->req.data[i] = responseDataBuf[j];

	 req->req.data[i] = 0;

	 //The last byte of data for SEND MESSAGE command is the check sum ,which is calculated
	 //from the next byte of the previous checksum that is the request slave address.
	 if (BmcIpmiVersion ==  IPMI_09_VERSION)
		 j = 0;
	 else 
	 {	
		if (BmcIpmiVersion ==	IPMI_10_VERSION)
			j = 1;
		else
			j = 2;
	 }
	for (; j < ( i -3); ++j)
		 req->req.data[i] += req->req.data[j+3];
	req->req.data[i]  = ~(req->req.data[i]) +1;
	++i;
	req->req.dataLength     = i;
	
	//The flags & timeouts are used by the driver internally.
	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units




    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);

	if( status) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}







	

	if ( 
		(respLength != 1) || (responseData[0] != 0)
					
		)

		{

		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;

		}



		
	
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// SendTimedLanMessageResponse
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedLanMessageResponse 
//  Purpose:    This function sends a response message to the DPC Over Lan
//  Context:     
//  Returns:    OK  else error status code
//  Parameters: 
//  
//  Notes:      none
*F*/




ACCESN_STATUS
SendTimedLanMessageResponse (

		ImbPacket *      ptr,           // pointer to the original request from EMP
		char      *responseDataBuf,
		int               responseDataLen,
		int             timeOut         // how long to wait, in mSec units
	)
{
	BOOL                            status;


    BYTE                                responseData[MAX_IMB_RESP_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int i,j;
	int oldId;

	//
	// If the Imb driver is not present return AccessFailed
	//
    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }



    while(!DriverReady)
     delay(400);




	//form the response packet first
	

	req->req.rsSa           =  BMC_SA;
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.cmd            =  WRITE_EMP_BUFFER;
	else 
		req->req.cmd            =  SEND_MESSAGE;        

	req->req.netFn          =  APP_NETFN;

	// After discussion with firmware team (Shailendra), the lun number needs to stay at 0
	// even though the DPC over Lan firmware EPS states that the lun should be 1 for DPC
	// Over Lan. - Simont (5/17/00)
	req->req.rsLun          =  0;

	i = 0;
	if (BmcIpmiVersion !=   IPMI_09_VERSION)
		req->req.data[i++]      = LAN_CHANNEL;

	req->req.data[i++]    =  ptr->rqSa;
	req->req.data[i++]      =  (((ptr->nfLn & 0xfc) | 0x4) | ((ptr->seqLn) & 0x3));
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.data[i++]    = ((~(req->req.data[0] +  req->req.data[1])) +1);
	else
		req->req.data[i++]    = ((~(req->req.data[1] +  req->req.data[2])) +1);
	req->req.data[i++]      =  BMC_SA; //though software is responding, we have to
								   //provide BMC's slave address as responder
								   //address.
	
	req->req.data[i++]      = ( (ptr->seqLn & 0xfc) | (ptr->nfLn & 0x3) );

	req->req.data[i++]      = ptr->cmd;
	for ( j = 0 ; j < responseDataLen ; ++j,++i)
	   req->req.data[i] = responseDataBuf[j];

	 req->req.data[i] = 0;
	 if (BmcIpmiVersion ==  IPMI_09_VERSION)
		 j = 0;
	 else 
		 j = 1;

	for (; j < ( i -3); ++j)
		 req->req.data[i] += req->req.data[j+3];
	req->req.data[i]  = ~(req->req.data[i]) +1;
	++i;
	req->req.dataLength     = i;

	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units




    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);

	if( status) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}







	

	if ( 
		(respLength != 1) || (responseData[0] != 0)
					
		)

		{

		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;

		}



		
	
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// SendTimedLanMessageResponse_Ex
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendTimedLanMessageResponse_Ex 
//  Purpose:    This function sends a response message to the DPC Over Lan
//  Context:     
//  Returns:    OK  else error status code
//  Parameters: 
//  
//  Notes:      none
*F*/




ACCESN_STATUS
SendTimedLanMessageResponse_Ex (

		ImbPacket *      ptr,           // pointer to the original request from EMP
		char      *responseDataBuf,
		int               responseDataLen,
		int             timeOut,         // how long to wait, in mSec units
		BYTE		sessionHandle,	//This is introduced in IPMI1.5,this is required to be sent in 
									//send message command as a parameter,which is then used by BMC
									//to identify the correct DPC session to send the mesage to.
		BYTE		channelNumber	//There are 3 different channels on which DPC communication goes on
									//Emp - 1,Lan channel one - 6,Lan channel two(primary channel) - 7.
	)
{
	BOOL                            status;


    BYTE                                responseData[MAX_IMB_RESP_SIZE];
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	int i,j;
	int oldId;

	//
	// If the Imb driver is not present return AccessFailed
	//
    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }



    while(!DriverReady)
     delay(400);




	//form the response packet first
	

	req->req.rsSa           =  BMC_SA;
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.cmd            =  WRITE_EMP_BUFFER;
	else 
		req->req.cmd            =  SEND_MESSAGE;        

	req->req.netFn          =  APP_NETFN;

	// After discussion with firmware team (Shailendra), the lun number needs to stay at 0
	// even though the DPC over Lan firmware EPS states that the lun should be 1 for DPC
	// Over Lan. - Simont (5/17/00)
	req->req.rsLun          =  0;

	i = 0;
	
	//checking for the IPMI version & then assigning the channel number for Lan accordingly.
	//This is the 1st byte of the SEND MESSAGE command.
	if (BmcIpmiVersion ==	IPMI_10_VERSION)
		req->req.data[i++] 	= LAN_CHANNEL;
	else if (BmcIpmiVersion ==	IPMI_15_VERSION)
		req->req.data[i++] 	= channelNumber;

	//The second byte of data for SEND MESSAGE starts with session handle
	req->req.data[i++] = sessionHandle;

	//Then it is the response slave address for SEND MESSAGE.
	req->req.data[i++]    =  ptr->rqSa;

	//Then the net function + lun for SEND MESSAGE command.
	req->req.data[i++]      =  (((ptr->nfLn & 0xfc) | 0x4) | ((ptr->seqLn) & 0x3));
	
	//Here the checksum is calculated.The checksum calculation starts after the channel number.
	//so for the IPMI 1.5 version its a checksum of 3 bytes that is session handle,response slave 
	//address & netfun+lun.
	if (BmcIpmiVersion ==   IPMI_09_VERSION)
		req->req.data[i++]    = ((~(req->req.data[0] +  req->req.data[1])) +1);
	else
	{
		if (BmcIpmiVersion == IPMI_10_VERSION)
			req->req.data[i++]    = ((~(req->req.data[1] +  req->req.data[2])) +1);
        else
			req->req.data[i++]    = ((~(req->req.data[2]+  req->req.data[3])) +1);
	}

	//This is the next byte of the message data for SEND MESSAGE command.It is the request 
	//slave address.
	req->req.data[i++]      =  BMC_SA; //though software is responding, we have to
								   //provide BMC's slave address as responder
								   //address.
	
	//This is just the sequence number,which is the next byte of data for SEND MESSAGE
	req->req.data[i++]      = ( (ptr->seqLn & 0xfc) | (ptr->nfLn & 0x3) );

	//The next byte is the command like get software ID(00).
	req->req.data[i++]      = ptr->cmd;

	//after the cmd the data ,which is sent by DPC & is retrived using the get message earlier
	// is sent back to DPC.
	for ( j = 0 ; j < responseDataLen ; ++j,++i)
	   req->req.data[i] = responseDataBuf[j];

	 req->req.data[i] = 0;

	 //The last byte of data for SEND MESSAGE command is the check sum ,which is calculated
	 //from the next byte of the previous checksum that is the request slave address.
	 if (BmcIpmiVersion ==  IPMI_09_VERSION)
		 j = 0;
	 else 
	 {	
		if (BmcIpmiVersion ==	IPMI_10_VERSION)
			j = 1;
		else
			j = 2;
	 }
	for (; j < ( i -3); ++j)
		 req->req.data[i] += req->req.data[j+3];
	req->req.data[i]  = ~(req->req.data[i]) +1;
	++i;
	req->req.dataLength     = i;

	//The flags & timeouts are used by the driver internally
	req->flags      = 0;
	req->timeOut    = timeOut * 1000;       // convert to uSec units




    WaitOnLocalSemaphore (MismicAccessSemphr);  
	ImbOsStartReq(req->timeOut,NULL);
	status = SendImbRequest(req,resp,& respLength,NULL);
	ImbOsEndReq(NULL);
    SignalLocalSemaphore (MismicAccessSemphr);

	if( status) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}







	

	if ( 
		(respLength != 1) || (responseData[0] != 0)
					
		)

		{

		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;

		}



		
	
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}


/////////////////////////////////////////////////////////////////////////////////////////
//IsAsyncMessageAvailable
/////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       IsMessageAvailable
//  Purpose:    This function waits for an Async Message  
//
//  Context:    Used by Upper level agents access Asynchronous IMB based
//              messages   
//  Returns:    OK  else error status code
//  Parameters: 
//               eventId
//    
//  Notes:     This call will block the calling thread if no Async events are 
//                              are available in the queue.
//
*F*/
ACCESN_STATUS
IsAsyncMessageAvailable (unsigned int   eventId )
{

	int oldId;

     oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }


  // confirm that       app is not using a phoney Id
  if (  AsyncEventHandle  !=  eventId)
	  return ACCESN_ERROR;
  WaitOnLocalSemaphore(AsyncEventHandle) ;
  if (AsyncEventUnregistrationFlag)
	  AckAsyncEventUnregistrationFlag = 1;

  SetThreadGroupID(oldId); 
  return ACCESN_OK;


}




/////////////////////////////////////////////////////////////////////////////////////////
//RegisterForImbAsyncMessageNotification
/////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       RegisterForImbAsyncMessageNotification
//  Purpose:    This function Registers the calling application  
//                              for Asynchronous notification when a sms message
//                              is available with the IMB driver.                       
//
//  Context:    Used by Upper level agents to know that an async
//                              SMS message is available with the driver.  
//  Returns:    OK  else error status code
//  Parameters: 
//    handleId  pointer to the registration handle
//
//  Notes:      The calling application should use the returned handle to 
//              get the Async messages..
*F*/
ACCESN_STATUS
RegisterForImbAsyncMessageNotification (unsigned int *handleId)

{

	
  
	int oldId;

    oldId = SetThreadGroupID (GlobalThreadGroupId);

    if (SuspendActivityFlag) {
	SetThreadGroupID(oldId);
	return ACCESN_ERROR ;
    }


    while(!DriverReady)
     delay(400);


	//allow the app to register once only

	if(AsyncEventHandle )
		{
			  SetThreadGroupID(oldId); 
		      return ACCESN_ERROR;
		}

 //create event based semaphorobject for notification of the async event

	AsyncEventHandle = OpenLocalSemaphore (0);
 
	if (!AsyncEventHandle)
	{
		  SetThreadGroupID(oldId); 
		  return ACCESN_ERROR;
	}

	*handleId = (unsigned int) AsyncEventHandle;
	
	SetThreadGroupID(oldId); 
	return ACCESN_OK;
}





/////////////////////////////////////////////////////////////////////////////////////////
//UnRegisterForImbAsyncMessageNotification
/////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       UnRegisterForImbAsyncMessageNotification
//  Purpose:    This function un-registers the calling application  
//                              for Asynchronous notification when a sms message
//                              is available with the IMB driver.                       
//
//  Context:    Used by Upper level agents to un-register
//                              for  async. notification of sms messages  
//  Returns:    OK  else error status code
//  Parameters: 
//    handleId  pointer to the registration handle
//        iFlag         a value that indicates who requested NLM unload 
//                              0 - EMP requested shutdown action of server (and thereby NLM termination)
//                              1 - unload command issued for NLM at console
//
//  Notes:      
*F*/
ACCESN_STATUS
UnRegisterForImbAsyncMessageNotification (unsigned int handleId, int iFlag)

{

	


	

	if( handleId != (unsigned int)AsyncEventHandle ) 
		return ACCESN_ERROR;


	AsyncEventUnregistrationFlag = 1;
		
	if (iFlag)
	{
	
		if (AsyncEventHandle != 0)
			SignalLocalSemaphore(AsyncEventHandle);

	
		//wait for the Async Thread from registering application to
		//acknowledge before closing the semaphore
		while(!AckAsyncEventUnregistrationFlag)
				ThreadSwitch();
	}
	
	if (AsyncEventHandle != 0)
		CloseLocalSemaphore(AsyncEventHandle);
		
	AsyncEventHandle = 0;
	
	return ACCESN_OK;
	
}



void ImbAsyncOsNewWakeup(void *context)
{

	if ( AsyncEventHandle !=0)
		SignalLocalSemaphore(AsyncEventHandle);

}





////////////////////////////////////////////////////////////////////////////////
// SetShutDownCode
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SetShutDownCode
//  Purpose:    This function sets the global Shutdown code and Delay time
//  Context:    Used by Upper level agents to set shutdown code and delay time
//              
//  Returns:    void
//  Parameters: 
//              delayTime Time to delay before Power Off or Reset in 100 MilliSeconds
//      actionCode Action to take while Shutting down.
//  Notes:      none
*F*/
ACCESN_STATUS SetShutDownCode( int delayTime, int actionCode)
{

DelayTime = delayTime;
ActionCode = actionCode;
return ACCESN_OK;


}

////////////////////////////////////////////////////////////////////////////////
// AtOSShutdown
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       AtOSShutdown
//  Purpose:    This function sends command to the BMC to Reset or 
//                              Power Off the system.
//  Context:    Called by the Operating System while shutting down.
//              
//  Returns:    void
//  Parameters: 
				No parameters
//  Notes:      If the Shut down Action code is S_NO_ACTION this routine
//                              returns immediately
*F*/
void
AtOSShutdown()
{



IMBPREQUESTDATA request;

char   inputData[8],outputData[8],completionCode;    
int    respLength = 1;
int i,j = 0, sum = 0;
if (ActionCode == S_NO_ACTION) 
return;

request.cmdType =  SET_WATCHDOG_TIMER;
request.rsSa    =  BMC_CONTROLLER;
request.busType =  PUBLIC_BUS;
request.netFn   =  NET_FUNCTION_APPLICATION  ; 
request.rsLun   =  0;
request.data    =  inputData;
inputData[0]    = 0x84;
request.dataLength = 6;
  
switch(ActionCode) {
    case S_RESET  :


		inputData[1] = 0x1;
		break;




    case S_POWER_OFF:

	   inputData[1] = 0x2;
       break;        
	   
    default :
	return;
    }
inputData[2] = inputData[3]=0;
inputData[4] = (char) DelayTime;
inputData[5] = (char) (DelayTime >> 8);
if (
	SendTimedImbpRequest(
    &request,
    300, 
    &outputData, 
    &respLength,
    &completionCode) != ACCESN_OK)
return;



request.cmdType =  RESET_WATCHDOG_TIMER;
request.dataLength = 0;


	// just a delay loop, I put something in it to make sure it does not get optimized out
	for(i=0;i<400000;i++) 
	{
		for(j=0;j<300;j++) 
		{
			sum+=j;
		}
		sum+=i;
	}

SendTimedImbpRequest(
    &request,
    300, 
    &outputData, 
    &respLength,
    &completionCode) ;

}






 
//////////////////////////////////////////////////////////////////////
// CheckFunction
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       CheckFunction
//  Purpose:    ensure proper clean up at nlm unload
//  Context:    called by netware during nlm unload
//  Returns:    after proper clenup, this function 
//              returns 0 to the operating system
//  Parameters: none
//  Notes:      none
*F*/

int
CheckFunction ()
{
    int oldId = SetThreadGroupID (GlobalThreadGroupId);
	// no need to wait as all the dependent NLMs have unloaded.
	// (pai 5/21)
	
	// before we close the semaphore verify the handle is valid
	if (AsyncEventHandle != 0)
	{
		CloseLocalSemaphore(AsyncEventHandle);
		AsyncEventHandle = 0;
	}

    CleanSemaphorePool();
    PoolDisabled=1;
    SuspendActivityFlag = 1;

    while (SuspendActivityFlag != 2)
		delay(100);
    SetThreadGroupID (oldId);    
    return 0;
}  




/*******************************IMB MESSAGING SUPPORT *******************************
************************************************************************************/

//////////////////////////////////////////////////////////////////////
// ImbOsStartReq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbOsStartReq
//  Purpose:    This function takes care of OS Specific Serialization
//                              of BMC access and starts the Timeout timer.
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters: 
//              timeOut Timeout interval to be set in uSec
//              context OS Specific context     
//  Notes:      none
*F*/
void
ImbOsStartReq (
		DWORD   timeOut,        // in uSec units
		void *  context
	)
{
    while(!TimeOutThreadReady )
	ThreadSwitch();
    TimeOutFlag = 1;
    TimeOutValue = timeOut/1000;
    SignalLocalSemaphore (TimeOutSemphr);
}


//////////////////////////////////////////////////////////////////////
// ImbReqTimedOut
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbReqTimedOut
//  Purpose:    This function determines if the Timeout Timer has 
//                              timed out
//  Context:    called by functions which use the Timeout Timer.
//  Returns:    This function returns TRUE if the Timeout has expired
//                              else returns FALSE.
//  Parameters: 
//              context OS Specific context     
//  Notes:      none
*F*/
BOOL
ImbReqTimedOut( void *context )
{
	return (!TimeOutFlag);
}


//////////////////////////////////////////////////////////////////////
// DelayRetryFunc
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       DelayRetryFunc
//  Purpose:    This function inserts a delay into the execution of 
//                              current thread
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters: 
//              retryCount Number of times to be spin before sleeping
//              delayTime Delay interval to be used in uSec
//              context OS Specific context     
//  Notes:      none
*F*/
VOID
DelayRetryFunc (
		DWORD retryCount,       // number of times we've been called, not used here
		DWORD delayTime,        // time to delay (in uSecs)
		void *context           // OS dependent handle,  not used here
	)
{
	//DWORD msDelay;
   //     if(retryCount<8)
    //            return;
   //     msDelay=delayTime/1000; // convert from uSecs to mSecs.
	ThreadSwitch();
}

//////////////////////////////////////////////////////////////////////
// GetNextImbSeq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       GetNextImbSeq
//  Purpose:    This function gets the next Imb Sequence number
//  Context:    called by functions sending IMB commands to devices
//  Returns:    This function returns the next possible sequence number
//  Parameters: 
//              context OS Specific context     
//  Notes:      The Sequence number takes values from 0-255
*F*/
BYTE
GetNextImbSeq (
		void *context
	)
{
	return imbSeqNo++;
}

//////////////////////////////////////////////////////////////////////
// ImbIf
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbIf
//  Purpose:    This function returns a pointer to the currently 
//                              detected Hardware Interface
//  Context:    called by  routines Interfacing to the SMIC or KCS layer
//  Returns:    Returns a pointer to the currently detected Hardware 
//                              Interface
//  Parameters: 
//              context OS Specific context     
//  Notes:      
*F*/

ImbInterface *
ImbIf(
		void *context
	)
{
	return imbIf;
}


//////////////////////////////////////////////////////////////////////
// ImbOsEndReq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbOsEndReq
//  Purpose:    This function takes care of OS Specific Serialization
//                              of BMC access and kills the Timeout timer.
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters: 
//              context OS Specific context     
//  Notes:      none
*F*/
void
ImbOsEndReq (
		void *  context
	)
{
    if ( TimeOutFlag )    
	{
		if (TimeOutThreadId != -1)
			ResumeThread(TimeOutThreadId);
		TimeOutFlag = 0;
	}

}




///////////////////////////////////////////////////////////////////////////
// Async messaging support
///////////////////////////////////////////////////////////////////////////




//////////////////////////////////////////////////////////////////////
// ImbAsyncOsInit
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsInit
//  Purpose:    This function initializes the OS dependent parts of the 
//                              Async Queue
//  Context:    called by Async Initialization functions
//  Returns:    void
//  Parameters: 
//              asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
void
ImbAsyncOsInit (
		ImbAsyncQueueInfo *asyncQ
	)
{
	//
	// init the Async Q semaphore
	
	asyncQ->syncSem=OpenLocalSemaphore(1);

// init the Semaphore pool locking semaphore
   semaPool.LockPool=OpenLocalSemaphore(1);
   
// Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.
	WaitOnLocalSemaphore(semaPool.LockPool);
	semaPool.PoolCount=0;
	semaPool.BusyFlags=NULL;
	semaPool.PoolMembers=NULL;
	GrowPoolAtRate(INITIAL_POOL_SIZE);
	SignalLocalSemaphore(semaPool.LockPool);
	return;
}
//
// 
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncQueue
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncQueue
//  Purpose:    This function returns a pointer to the Async queue info
//  Context:    called by functions manipulating the Async Q
//  Returns:    returns a pointer to the Async queue info
//  Parameters: 
//              context OS Specific context     
//  Notes:      none
*F*/

ImbAsyncQueueInfo *
ImbAsyncQueue (
		void *context
	)
{
	return &asyncQ;
}
//
// 
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsReqWait
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsReqWait
//  Purpose:    This function makes the incoming thread wait for a new 
//                              SMS message to come in
//  Context:    called by Async functions
//  Returns:    void
//  Parameters: 
//              asyncQ  pointer to Async Queue Specific Info
//              timeOut Time to wait before timing out.
//  Notes:      none
*F*/
void
ImbAsyncOsReqWait (
		ImbAsyncQueueInfo *     asyncQ,
		DWORD                           timeOut
	)
{
	long semaphore;
	long msTimeOut;

	msTimeOut=timeOut/1000;
	WaitOnLocalSemaphore(semaPool.LockPool);        
	semaphore=GetALocalSemaphore();
	SignalLocalSemaphore(semaPool.LockPool);        
	
	if(TimedWaitOnLocalSemaphore(semaphore,msTimeOut)==ERR_TIMEOUT_FAILURE)
	{
		WaitOnLocalSemaphore(semaPool.LockPool);        
		ReleaseSpecificLocalSemaphore(semaphore);
		SignalLocalSemaphore(semaPool.LockPool);        
	}
	return;
}

//////////////////////////////////////////////////////////////////////
// ImbAsyncOsReqWakeup
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsReqWakeup
//  Purpose:    This function wakes up all sleeping threads waiting for a 
//                              message to come in
//  Context:    called by Async  functions
//  Returns:    void
//  Parameters: 
//              asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
void
ImbAsyncOsReqWakeup (
		ImbAsyncQueueInfo *     asyncQ
	)
{
	WaitOnLocalSemaphore(semaPool.LockPool);        
	SignalActiveLocalSemaphores();
	SignalLocalSemaphore(semaPool.LockPool);        
	return;
}

//////////////////////////////////////////////////////////////////////
// ImbAsyncOsLock
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsLock
//  Purpose:    This function obtains exclusive use of the async message 
//                              queue
//  Context:    called by Async  functions
//  Returns:    returns true if able to obtain exclusive use of Async Q
//  Parameters: 
//              asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
BOOL
ImbAsyncOsLock (
		ImbAsyncQueueInfo *asyncQ
	)
{
	return(!WaitOnLocalSemaphore(asyncQ->syncSem));
}
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsUnLock
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsUnLock
//  Purpose:    This function releases exclusive use of the async message 
//                              queue
//  Context:    called by Async  functions
//  Returns:    returns true if able to release exclusive use of Async Q
//  Parameters: 
//              asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
BOOL
ImbAsyncOsUnLock (
		ImbAsyncQueueInfo *asyncQ
	)
{
	SignalLocalSemaphore(asyncQ->syncSem);
	return TRUE;
}





// MapPhysicalMemory 
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       MapPhysicalMemory 
//  Purpose:    This function maps specified range of physical memory in calling 
//              pocesse's address space
//  Context:    Used by Upper level agents (sis modules) to access system physical memory 
//  Returns:    ACCESN_OK  else error status code
//  Parameters: 
//     
//     startAddress   starting physical address of the  memory to be mapped 
//
//     addressLength  length of the physical memory to be mapped
//
//     virtualAddress pointer to the mapped virtual address
//
//  Notes:      right now only one address range can be mapped at a time.
//              before a new address range is mapped, the previously mapped 
//                range must be unmapped.
*F*/

int MapPhysicalMemory(int startAddress,int addressLength, int *virtualAddress )

{

#if 0
     if (!addressLength || ApiMapVirtualAddress)
     {
	ConsolePrintf ("\n 1");     
	return (int)MEMORY_MAP_FAILURE;
     }
#endif
	 int index;

    if (!addressLength )
    {
		return (int)MEMORY_MAP_FAILURE;
    }

 	for (index = 0; index < MAX_MAPPED_ADDR; index++)
	{
		if ( aApiMapVirtualAddress[index] == 0)
		{
			break;
		}
	}

	 if (index >= MAX_MAPPED_ADDR)
	 {
		return (int)MEMORY_MAP_FAILURE;
	 }

    *virtualAddress = (int)  malloc(addressLength *sizeof(char));
    if ((*virtualAddress) == (int) NULL)
    {

	return (int) MEMORY_MAP_FAILURE;

    }


    if (CopyPhysicalMemory((BYTE*)startAddress,(BYTE*)*virtualAddress,(long)addressLength))
    {
       //ApiMapVirtualAddress = *virtualAddress;
	   aApiMapVirtualAddress[index] = *virtualAddress;
       return (int)ACCESN_OK ;
    }
    else
	free((char *) *virtualAddress);


     return (int)MEMORY_MAP_FAILURE ;      
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// CopyPhysicalMemory
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       CopyPhysicalMemory
//  Purpose:    This function maps the physical address to process specific
//                              logical address and copies this address into the given buffer
//  Context:    Used by MapPhysicalMemory
//  Returns:    1 if Successful 0 if failure
//  Parameters: 
//     
//     pSource  pointer to the physical address
//         pDestination pointer to Logical address
//  Notes:      none
*F*/
int CopyPhysicalMemory(BYTE* pSource, BYTE* pDestination, LONG lNumUnits)
{
    LONG lLogicalSource = 0;

    lLogicalSource = MapAbsoluteAddressToDataOffset(pSource);    

    if(lLogicalSource == 0)
	return 0;

    memcpy(pDestination, (char*)lLogicalSource, lNumUnits);
	    
    return 1;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// UnmapPhysicalMemory 
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       UnMapPhysicalMemory 
//  Purpose:    This function unmaps the previously mapped physical memory
//  Context:    Used by Upper level agents (sis modules)  
//  Returns:    ACCESN_OK  else error status code
//  Parameters: 
//     
//     virtualAddress  pointer to the mapped virtual address
//
//  Notes:      none
*F*/
int UnmapPhysicalMemory( int virtualAddress )
 {
#if 0
	if( virtualAddress != ApiMapVirtualAddress )
	   return (int) MEMORY_UNMAP_FAILURE ;
	free((char *)virtualAddress);
	ApiMapVirtualAddress = 0;
	return (int) ACCESN_OK;
#endif

	int i;
	
    for (i=0; i<MAX_MAPPED_ADDR; i++)
	{
		if( virtualAddress == aApiMapVirtualAddress[i])
		{
			free((char *)virtualAddress);		
			aApiMapVirtualAddress[i] = 0;
			return (int) ACCESN_OK;
		}
	}
 	return (int) MEMORY_UNMAP_FAILURE ;
 }

/////////////////////////////////////////////////////////////////////////////////////////
// PerformCleanup
/////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       PerformCleanup
//  Purpose:    It performs memory clean up 
//  Context:    This function is called while unloading the mismic driver
//  Returns:    None
//  Parameters: 
//     None
//  Notes:      none
*F*/
void 
PerformCleanup (void)
{
#if 0
    if ( ApiMapVirtualAddress)
		free((void *)ApiMapVirtualAddress);
#endif
	int i;
	
    for (i=0; i<MAX_MAPPED_ADDR; i++)
	{
		if( aApiMapVirtualAddress[i])
		{
			free((char *)aApiMapVirtualAddress[i]);
		}
	}

}

/*
	The following functions are used to implement the Async message queueing 
	mechanisms.When a Higher level thread comes down to get an async message
	it is put to sleep if there are no messages in the Async Q.When the main 
	thread loops around and puts a new message into the Async Q, it also has
	to wake up all these sleeping threads.For this to happen one has to use 
	a notification mechanism to ask these threads to wakeup and retreive the
	new message.Under NT this is implemented by using Notification events.
	Since netware does not have anything remotely close to Notification events
	we have used a pool of semaphores to implement this mechanism.At init 
	time an array of possible semaphores is initialized.Each element in this
	array has a corresponding element in another array called BusyFlags.The 
	purpose of this second array is to keep track of which element in the
	first array is being used.When a thread needs to be put to sleep the Async 
	code goes through the second array to figure out which element is not used
	and the empty entry is allocated with a new semaphore handle and the thread
	is put to sleep on that semaphore.If no empty entry is found it grows both
	the arrays , and allocates an empty entry.When a new async message comes
	in the BusyFlags array is scanned and all the active semaphores are signalled.
		- Fizal
*/


// Warning:Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.Obtain the lock by calling 
// WaitOnLocalSemaphore(semaPool.LockPool) and release it by calling
// SignalLocalSemaphore(semaPool.LockPool)

int GetALocalSemaphore()
{
	int i;
	int semaIndex=-1;
	if(PoolDisabled)
		return 0;
	for(i=0;i<semaPool.PoolCount;i++)
	{
		if(!semaPool.BusyFlags[i])
		{
			semaIndex=i;
			break;
		}
	}
	if(semaIndex!=-1)
	{
		// found an empty slot
		if(!semaPool.PoolMembers[semaIndex])
			semaPool.PoolMembers[semaIndex]=OpenLocalSemaphore(0);
		semaPool.BusyFlags[semaIndex]=1;
		return semaPool.PoolMembers[semaIndex];
	}
	else
	{
		//grow the pool
		GrowPoolAtRate((unsigned int)4);
		return (GetALocalSemaphore());
	}
}

// Warning:Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.Obtain the lock by calling 
// WaitOnLocalSemaphore(semaPool.LockPool) and release it by calling
// SignalLocalSemaphore(semaPool.LockPool)

void GrowPoolAtRate(unsigned int poolGrowthRate)
{
	char *tempBusy;
	long *tempMember;

	tempBusy=malloc((semaPool.PoolCount+poolGrowthRate)*sizeof(char));
	tempMember=malloc((semaPool.PoolCount+poolGrowthRate)*sizeof(long));

	memset(tempBusy,0,(semaPool.PoolCount+poolGrowthRate)*sizeof(char));
	memset(tempMember,0,(semaPool.PoolCount+poolGrowthRate)*sizeof(long));

	if(semaPool.BusyFlags)
	{
		memcpy(semaPool.BusyFlags,tempBusy,semaPool.PoolCount*sizeof(char));
		free(semaPool.BusyFlags);
	}
	if(semaPool.PoolMembers)
	{
		memcpy(semaPool.PoolMembers,tempMember,semaPool.PoolCount*sizeof(long));
		free(semaPool.PoolMembers);
	}
	semaPool.BusyFlags=tempBusy;
	semaPool.PoolMembers=tempMember;
}

// Warning:Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.Obtain the lock by calling 
// WaitOnLocalSemaphore(semaPool.LockPool) and release it by calling
// SignalLocalSemaphore(semaPool.LockPool)
void SignalActiveLocalSemaphores()
{
	int i;
	if(PoolDisabled)
		return;
	for(i=0;i<semaPool.PoolCount;i++)
	{
		if(semaPool.BusyFlags[i])
		{
			SignalLocalSemaphore(semaPool.PoolMembers[i]);
			semaPool.BusyFlags[i]=0;
		}
	}
}
// Warning:Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.Obtain the lock by calling 
// WaitOnLocalSemaphore(semaPool.LockPool) and release it by calling
// SignalLocalSemaphore(semaPool.LockPool)
void ReleaseSpecificLocalSemaphore(long LocalSemaphore)
{
	int i;
	if(PoolDisabled)
		return;
	for(i=0;i<semaPool.PoolCount;i++)
	{
		if(semaPool.PoolMembers[i]==LocalSemaphore)
			semaPool.BusyFlags[i]=0;
	}
}
// Warning:Anybody manipulating the Semaphore pool has to obtain the lock
// The functions manipulating the Semaphore pool assume that the caller 
// has obtained the Lock.Obtain the lock by calling 
// WaitOnLocalSemaphore(semaPool.LockPool) and release it by calling
// SignalLocalSemaphore(semaPool.LockPool)
void CleanSemaphorePool()
{
	int i;


	if(PoolDisabled)
		return ;
	for(i=0;i<semaPool.PoolCount;i++)
	{
		if(semaPool.BusyFlags[i])
		{
		// wake up any waiting thread
			SignalLocalSemaphore(semaPool.PoolMembers[i]);
		}
		// destroy the semaphore
		if(semaPool.PoolMembers[i])
		{

			CloseLocalSemaphore(semaPool.PoolMembers[i]);
			semaPool.PoolMembers[i] = 0; // reset it to 0
		}
	}
	// destroy the semaphore pool
	free(semaPool.PoolMembers);
	free(semaPool.BusyFlags);
}

/////////////////////////////////////////////////////////////////////////////////////////
//SendAsyncImbpRequest
/////////////////////////////////////////////////////////////////////////////////////////
//
// Send an IMB request that we don't expect to get a response to
//
/////////////////////////////////////////////////////////////////////////////////////////////////
// SendAsyncImbpRequest
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SendAsyncImbpRequest
//  Purpose:    This function sends a request for Asynchronous IMB implemented function
//  Context:    Used by Upper level agents (sis modules) to access Asynchronous IMB implemented functionality. 
//  Returns:    OK  else error status code
//  Parameters: 
//     reqPtr  Pointer to Async IMB request
//     seqNo   Sequence Munber
//  Notes:      none
*F*/
ACCESN_STATUS
SendAsyncImbpRequest (
		IMBPREQUESTDATA *reqPtr,                // request info and data
		BYTE *                  seqNo                   // IMB sequence number used when creating IMB msg
	)
{
	BOOL                            status;
	int oldId;      

    BYTE                                responseData[MAX_IMB_RESP_SIZE];

	
	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
	DWORD                           respLength = sizeof( responseData );

    BYTE                                requestData[MAX_IMB_RESP_SIZE];
	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
	//
	// If the Imb driver is not present return AccessFailed
	//
	oldId = SetThreadGroupID (GlobalThreadGroupId); 
	req->req.rsSa           = reqPtr->rsSa;
	req->req.cmd            = reqPtr->cmdType;
	req->req.netFn          = reqPtr->netFn;
	req->req.rsLun          = reqPtr->rsLun;
	req->req.dataLength     = reqPtr->dataLength;

	memcpy( req->req.data, reqPtr->data, reqPtr->dataLength );

	req->flags              = NO_RESPONSE_EXPECTED;
	req->timeOut    = 0;    // no timeouts for async sends

    WaitOnLocalSemaphore (MismicAccessSemphr);  
	TimeOutFlag=1;
	status = SendImbRequest(req,resp,& respLength,NULL);
	TimeOutFlag=0;
    SignalLocalSemaphore (MismicAccessSemphr);


	if( status ) {
		SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}
	if( respLength != 2 ) {
	       SetThreadGroupID(oldId); 
		return ACCESN_ERROR;
	}
	//
	// give the caller his sequence number
	//
	*seqNo = resp->data[0];
	SetThreadGroupID(oldId); 
	return ACCESN_OK;
}
/////////////////////////////////////////////////////////////////////////////////////////
//GetAsyncImbpMessage
/////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       GetAsyncImbpMessage
//  Purpose:    This function gets the next available async message with a message id
//                              greater than SeqNo. The message looks like an IMB packet
//                              and the length and Sequence number is returned
//  Context:    Used by Upper level agents (sis modules) to access Asynchronous IMB implemented functionality. 
//  Returns:    OK  else error status code
//  Parameters: 
//     msgPtr  Pointer to Async IMB request
//         msgLen  Length 
//         timeOut Time to wait 
//  seqNo   Sequence Munber
//  Notes:      none
*F*/
ACCESN_STATUS
GetAsyncImbpMessage (
						ImbPacket *             msgPtr,                                 // request info and data
						DWORD *                 msgLen,                                 // IN - length of buffer, OUT - msg len
						DWORD                   timeOut,                                // how long to wait for the message
						ImbAsyncSeq *                   seqNo,                                  // previously returned seq number (or ASYNC_SEQ_START)
						DWORD                                   channelNumber
					)
{

	BOOL            status;
	int                     oldId;

    BYTE                responseData[MAX_ASYNC_RESP_SIZE],lun;
	ImbAsyncResponse *      resp = (ImbAsyncResponse *) responseData;
	DWORD           respLength = sizeof( responseData );

	ImbAsyncRequest         req;


	oldId = SetThreadGroupID (GlobalThreadGroupId);  
	//
	// If the Imb driver is not present return AccessFailed
	//
	if( msgPtr == NULL || msgLen == NULL || seqNo == NULL )
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	req.timeOut             = timeOut * 1000;       // convert to uSec units
	req.lastSeq             = *seqNo;

	status = ImbAsyncGetMessage( &req, resp, &respLength, &asyncQ );
	if(status)
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	if( respLength < MIN_ASYNC_RESP_SIZE ) 
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}
	respLength -= MIN_ASYNC_RESP_SIZE;

	if( *msgLen < respLength ) 
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	// this change was moved from imbapi.c to this file - shankar
	//@@@@@
	if ( BmcIpmiVersion == IPMI_09_VERSION)
	{

		switch( channelNumber)
		{

			case IPMB_CHANNEL:

				lun = IPMB_LUN;
				 break;

			case  EMP_CHANNEL:
				lun = EMP_LUN;
				  break;

			default:
				
				lun = RESERVED_LUN;
				 break;


		}

		if ( (lun == RESERVED_LUN) || (lun !=  ((((ImbPacket *)(resp->data))->nfLn) & 0x3 )) )
		{
			*seqNo = resp->thisSeq;
									
		}
		memcpy( msgPtr, resp->data, respLength );
		*msgLen = respLength;
				
			
	}
	else 
	{
		//it's a 1.0 or  above version  

		if (resp->data[0] != (BYTE)channelNumber)
		{
			*seqNo = resp->thisSeq;
			SetThreadGroupID(oldId);  
			return ACCESN_ERROR;
			
		}

		memcpy( msgPtr, &(resp->data[1]), respLength-1 );
		*msgLen = respLength-1;
		

	}
	//@@@@@



	//memcpy( msgPtr, resp->data, respLength );
	//*msgLen = respLength;

	//
	// give the caller his sequence number
	//
	*seqNo = resp->thisSeq;
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}

/////////////////////////////////////////////////////////////////////////////////////////
//GetAsyncImbpMessage_Ex
/////////////////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       GetAsyncImbpMessage_Ex
//  Purpose:    This function gets the next available async message with a message id
//                              greater than SeqNo. The message looks like an IMB packet
//                              and the length and Sequence number is returned
//  Context:    Used by Upper level agents (sis modules) to access Asynchronous IMB implemented functionality. 
//  Returns:    OK  else error status code
//  Parameters: 
//     msgPtr  Pointer to Async IMB request
//         msgLen  Length 
//         timeOut Time to wait 
//  seqNo   Sequence Munber
//  Notes:      none
*F*/
ACCESN_STATUS
GetAsyncImbpMessage_Ex (
						ImbPacket *             msgPtr,                                 // request info and data
						DWORD *                 msgLen,                                 // IN - length of buffer, OUT - msg len
						DWORD                   timeOut,                                // how long to wait for the message
						ImbAsyncSeq *                   seqNo,                                  // previously returned seq number (or ASYNC_SEQ_START)
						DWORD                                   channelNumber,
						BYTE *					sessionHandle,
						BYTE *					privilege
					)
{

	BOOL            status;
	int                     oldId;

    BYTE                responseData[MAX_ASYNC_RESP_SIZE],lun;
	ImbAsyncResponse *      resp = (ImbAsyncResponse *) responseData;
	DWORD           respLength = sizeof( responseData );

	ImbAsyncRequest         req;


	oldId = SetThreadGroupID (GlobalThreadGroupId);  
	//
	// If the Imb driver is not present return AccessFailed
	//
	if( msgPtr == NULL || msgLen == NULL || seqNo == NULL )
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	req.timeOut             = timeOut * 1000;       // convert to uSec units
	req.lastSeq             = *seqNo;

	status = ImbAsyncGetMessage( &req, resp, &respLength, &asyncQ );
	if(status)
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	if( respLength < MIN_ASYNC_RESP_SIZE ) 
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}
	respLength -= MIN_ASYNC_RESP_SIZE;

	if( *msgLen < respLength ) 
	{
		SetThreadGroupID(oldId);  
		return ACCESN_ERROR;
	}

	// this change was moved from imbapi.c to this file - shankar
	//@@@@@
	if ( BmcIpmiVersion == IPMI_09_VERSION)
	{

		switch( channelNumber)
		{

			case IPMB_CHANNEL:

				lun = IPMB_LUN;
				 break;

			case  EMP_CHANNEL:
				lun = EMP_LUN;
				  break;

			default:
				
				lun = RESERVED_LUN;
				 break;


		}

		if ( (lun == RESERVED_LUN) || (lun !=  ((((ImbPacket *)(resp->data))->nfLn) & 0x3 )) )
		{
			*seqNo = resp->thisSeq;
									
		}
		memcpy( msgPtr, resp->data, respLength );
		*msgLen = respLength;
				
			
	}
	else 
	{
		if((sessionHandle ==NULL) || (privilege ==NULL))
			return ACCESN_ERROR;

		//With the new IPMI version the get message command returns the 
		//channel number along with the privileges.The 1st 4 bits of the
		//second byte of the response data for get message command represent
		//the channel number & the last 4 bits are the privileges.
		*privilege = (resp->data[0] & 0xf0)>> 4;
		if ((resp->data[0] & 0x0f) != (BYTE)channelNumber)
		{
			*seqNo = resp->thisSeq;
			SetThreadGroupID(oldId);  
			return ACCESN_ERROR;
			
		}
		
		//The get message command according to IPMI 1.5 spec now even
		//returns the session handle.This is required to be captured
		//as it is required as request data for send message command.
		*sessionHandle = resp->data[1];
		memcpy( msgPtr, &(resp->data[1]), respLength-1 );
		*msgLen = respLength-1;
		

	}
	//@@@@@



	//memcpy( msgPtr, resp->data, respLength );
	//*msgLen = respLength;

	//
	// give the caller his sequence number
	//
	*seqNo = resp->thisSeq;
	SetThreadGroupID(oldId);  
	return ACCESN_OK;
}






void getBmcIpmiVersion()
{



	
	unsigned char   imbReq[128]     ;
	unsigned char   imbResp[128]    ;
	ImbRequestBuffer  * reqBuffer;
	ImbResponseBuffer * respBuffer;

	DWORD                           respLength;
	reqBuffer   =           (ImbRequestBuffer *)imbReq;
	respBuffer  =           (ImbResponseBuffer *) imbResp; 
    
	reqBuffer->req.rsSa                     = BMC_SA;
	reqBuffer->req.cmd                      = GET_DEVICE_ID;
	reqBuffer->req.netFn                    = APP_NETFN;
	reqBuffer->req.rsLun                    = 0;
	reqBuffer->req.dataLength               = 0;
    reqBuffer->flags                        = 0;
	reqBuffer->timeOut                      = 500; 

	respLength                              = MAX_BMC_RESPONSE_SIZE;

	ImbOsStartReq(reqBuffer->timeOut,NULL);


   // This is a chicken and egg problem. To determine IPMI version we 
   // need to send a command to BMC. But to send a command to BMC we
   // have to know the version of IPMI.
   // So we try by faking  OLD version first (pai 11/15/99)

   BmcIpmiVersion  = IPMI_OLD_VER;
   if (
		(SendImbRequest( reqBuffer, 
						respBuffer,
						& respLength,
						NULL
					  ) == IMB_SUCCESS )  && (!respBuffer->cCode) &&
						( respLength < (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1)) 
	  )    {
			        ConsolePrintf("ipmi 0.9 \n");
		
				;       //It is the old IPMI version 
						 
			} else {
						// check for 1.0 version since check for old version failed
						ImbOsEndReq(NULL);
						delay(500);
						// reset the buffers
						
						reqBuffer   =           (ImbRequestBuffer *)imbReq;
						respBuffer  =           (ImbResponseBuffer *) imbResp; 
    
						reqBuffer->req.rsSa                     = BMC_SA;
						reqBuffer->req.cmd                      = GET_DEVICE_ID;
						reqBuffer->req.netFn                    = APP_NETFN;
						reqBuffer->req.rsLun                    = 0;
						reqBuffer->req.dataLength               = 0;
						reqBuffer->flags                        = 0;
						reqBuffer->timeOut                      = 500; 

						respLength                              = MAX_BMC_RESPONSE_SIZE;


						BmcIpmiVersion = IPMI_1_VER;
						ImbOsStartReq(reqBuffer->timeOut,NULL);
						if (
							 (      SendImbRequest( reqBuffer, 
												respBuffer,
												& respLength,
												NULL
											)== IMB_SUCCESS
							 )      &&      (!respBuffer->cCode) 
								&&      ( respLength >= (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1)) 

			   )    {

								      
										switch (respBuffer->data[4])
					  
												{
					  
									 
													case IPMI_1_VER:
														BmcIpmiVersion = (unsigned char)IPMI_1_VER;
														ConsolePrintf("ipmi 1.0 \n");    

														break;
													case IPMI_15_VER://IPMI 1.5 --svuppula
														BmcIpmiVersion = (unsigned char)IPMI_15_VER;
														break;


													 //-----------------------------------------------------------------
													 // Future IPMI versions here.
													 //-----------------------------------------------------------------


													default:
														//right now assume that default version is 1.0

													BmcIpmiVersion = (unsigned char)IPMI_1_VER;
													 	ConsolePrintf("ipmi 1.0 \n");    
														break;

										
												} 
									
	
									}  
									else
									{

												BmcIpmiVersion = IPMI_UNKNOWN_VER;
												ConsolePrintf(" IPMI version 1.0 or 0.9 has not been detected \n");

									}
				
				}

		ImbOsEndReq(NULL);      
	}




			










int isIpmiOldVersion(void *context)
{

if (BmcIpmiVersion == (unsigned char)IPMI_OLD_VER )
return 1;
else
return 0;

}



/////////////////////////////////////////////////////////////////////////////////////////////////
// GetIpmiVersion
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       GetIpmiVersion 
//  Purpose:    This function returns current IPMI version
//  Context:   
//  Returns:    IPMI version
//  Parameters: 
//     reqPtr
//     timeOut
//     respDataPtr
//     respLen
//  Notes:      svuppula
*F*/
BYTE	GetIpmiVersion()
{
	return	BmcIpmiVersion;
}
