
/*M*                     
//                    INTEL CORPORATION PROPRIETARY INFORMATION
//      This software is supplied under the terms of a license agreement or
//      nondisclosure agreement with Intel Corporation and may not be copied
//      or disclosed except in accordance with the terms of that agreement.
//            Copyright (c) 1995 Intel Corporation.  All Rights Reserved.
//
*M*/

#include "imb_os.h"
#include "smbios.h"
#include "imbapi.h"
#if 0
#include "debugprint.h"
#include "diagcodes.h"
#include "diagmsg.h"
#endif
#include "imb_drv.h"

extern ImbInterface *ImbInterfaces[];


/*F*
//*****************************************************************************
//* ScanForIPMIDeviceInfo(...)
//*****************************************************************************
//* Name:       ScanForIPMIDeviceInfo
//* Purpose:    Scans the SMBIOS table for IPMI Device Information record and
//*				returns the same.
//* Context:    Called by NT kernel at boot time.
//* Returns:    NTSTATUS, standard NT DDK defined status codes (STATUS_SUCCESS
//*             or error code)
//* Parameters:
//*				pDeviceExtension	 IN - Device Extension.
//*				paSmBiosTableAddress IN - Physical Address of SMBIOS table.
//*				ulSmBiosTableLength  IN - SMBIOS Table Length
//* Notes:      none
//*****************************************************************************
*F*/

BOOL ScanForIPMIDeviceInfo ( 	struct ImbIfStruct **pImpIf, 
								UCHAR 	*paSmBiosTableAddress,
							  	ULONG   ulSmBiosTableLength)
{
	int     i;
 	ULONG   ulOffset = 0, ulOrgMemLength;
	PUCHAR  pOrgVirtualAddress = NULL, pVirtualAddress = NULL, pucTemp = NULL;
	BOOL 	bStatus = FALSE;
	
	PSMBIOSHeader 		   pSmBiosInfo = NULL;
	PIPMIDeviceInformation pIpmiDevInfo = NULL;

 	do
  	{
		//
		// Check if the SMBIOS table address and length are valid...
		//
    	if (!paSmBiosTableAddress || (ulSmBiosTableLength <= 0))
    	{
    		ConsolePrintf ( "Invalid Physical Address and Range of Bios Table.");
			return bStatus;
    	}
		//
		// If the table address is not in 64K boundary - is usually isn't -
		// perform offset manipulation.
		//
		if ((ULONG)paSmBiosTableAddress & BYTE_BOUNDARY_64K)
		{
      		ulOffset = ((ULONG)paSmBiosTableAddress & BYTE_BOUNDARY_64K);
      		(ULONG)paSmBiosTableAddress &= ~((DWORD)BYTE_BOUNDARY_64K);
			ulSmBiosTableLength += ulOffset;
		}
		//
		// Map the physical address to virtual memory...
		//
        //ntStatus = MapPhysicalToVirtualMem(pDeviceExtension, paSmBiosTableAddress, &ulSmBiosTableLength, &pVirtualAddress);
	    ulOrgMemLength = ulSmBiosTableLength;
#if 0
		currentIrql = KeGetCurrentIrql();
        if (currentIrql != PASSIVE_LEVEL)
		{
			D_DiagMsg1(D_Errors,"IRQ level is too high: %x",currentIrql);
			// sundar break;
			return bStatus;
		}
#endif
		if (MapPhysicalMemory ((int)paSmBiosTableAddress, ulSmBiosTableLength, (int *)&pVirtualAddress) 
						!=  ACCESN_OK )
		{
			ConsolePrintf ("No Virtual Pointer to SMBios Table.");
			ConsolePrintf ("%x : %d %x \n",
							paSmBiosTableAddress, ulSmBiosTableLength, pVirtualAddress);
			// sundar break;
			return bStatus;
		}
		pOrgVirtualAddress = pVirtualAddress;
		
	    pVirtualAddress += ulOffset;
		ulSmBiosTableLength -= ulOffset;
		while (ulSmBiosTableLength > 0)
		{
    //D_DiagMsg3(D_MidLevelInfo,"Searching Record # %x Length:%x, Remaining: %x", ((PSMBIOSHeader)pVirtualAddress)->Type, ((PSMBIOSHeader)pVirtualAddress)->Length, ulSmBiosTableLength);

			pSmBiosInfo = (PSMBIOSHeader) pVirtualAddress;
			
			if (IPMI_DEVICE_INFORMATION == pSmBiosInfo->Type)
			{
    		    ConsolePrintf ( "Located IPMI Device Information Record...");
				
				pIpmiDevInfo = (PIPMIDeviceInformation)pVirtualAddress;
                
				switch(pIpmiDevInfo->itInterfaceType)
        		{
        			case    KCS:
				          	ConsolePrintf ("KCS Interface detected. Setting up...");
				            *pImpIf = ImbInterfaces[IMB_INTERFACE_KCS];
				            break;
		         	case    SMIC:
        				  	ConsolePrintf ("SMIC Interface detected. Setting up...");
                        	*pImpIf = ImbInterfaces[IMB_INTERFACE_SMIC];
                        	break;
                    case    Unknown:
                    case    BT:
                    default:
                        ConsolePrintf ("BT/Unknown Interface NOT supported.");
                        break;
                }
                if (pIpmiDevInfo->BaseAddress & IO_MAPPED_ADDRESS)
                {
#if 0
                    ConsolePrintf ("Base Address is I/O Mapped.");
                    (*pImpIf)->DefaultBaseAddress = \
                        (pIpmiDevInfo->BaseAddress & ~((ULONG)IO_MAPPED_ADDRESS));
                    (*pImpIf)->DefaultBaseAddress |= (pIpmiDevInfo->bBamLSBitAddress);
                    switch(pIpmiDevInfo->bBamRegisterSpacing)
                    {
                        case    BYTE_BOUNDARY:
                            (*pImpIf)->bmcRegSpacing = 1;
                            break;
                        case    BIT32_BOUNDARY:
                            (*pImpIf)->bmcRegSpacing = 4;
                            break;
                        case    BYTE16_BOUNDARY:
                            (*pImpIf)->bmcRegSpacing = 16;
                            break;
                        default:
                            ConsolePrintf ("Register Spacing Value is Invalid. Assuming default");
                            (*pImpIf)->bmcRegSpacing = 1;
                            break;
                    }
#endif
                }
                bStatus = TRUE;
				break;
			}
            ulSmBiosTableLength -= pSmBiosInfo->Length;
            pVirtualAddress += pSmBiosInfo->Length;
/*
			// IMPORTANT: This code to skip strings attached to the end of records
			// works only for 32-bit environment, referencing a pointer on 64-bit
			// without proper alignment, causes Kernel Fix-up to be called and found
			// to cause system freeze.
            for(i=0;(*((PUSHORT)((pVirtualAddress+i)) && ((ULONG)i < ulSmBiosTableLength));i++)
			{
                //scasb till double null or end of smbios table length.
			}
*/
			// Scan through the array of strings and reach the delimiter.
#if 1
            for(i=0,pucTemp = pVirtualAddress;(ULONG)i < ulSmBiosTableLength;i++)
            {
                if (!(pucTemp[i] || pucTemp[i+1]))
                {
                    break;
                }
            }
#endif

#if 0
			for (i=0, pucTemp = pVirtualAddress; 
							(pucTemp[i] || pucTemp[i+1]) && (i < ulSmBiosTableLength);
							i++);
#endif
			ulSmBiosTableLength -= (i+2);
            pVirtualAddress += (i+2);
			//Loop again till the end of smbios table is reached.
		}	// while
	}		
    while (FALSE);
    UnmapPhysicalMemory((int)pOrgVirtualAddress);
    return bStatus;
}

/*F*
//*****************************************************************************
//* GetIPMIDeviceInformation(...)
//*****************************************************************************
//* Name:       GetIPMIDeviceInformation
//* Purpose:    This function searches for SMBIOS entry point, obtains the
//*				table address and calls ScanForIpmiDeviceInfo to get Rec#38
//*				
//* Context:    Called by NT kernel at boot time.
//* Returns:    NTSTATUS, standard NT DDK defined status codes (STATUS_SUCCESS
//*             or error code)
//* Parameters:
//*             pDeviceExtension            OUT - Filled IPMI Device Information Record in Device Extn.
//* Notes:      none
//*****************************************************************************
*F*/
BOOL GetIPMIDeviceInformation (struct ImbIfStruct **pImpIf)
{
	BOOL 	bStatus = FALSE;
	PUCHAR 	paSmBiosAddress, paSmTableAddress;
 	ULONG   ulLength, ulTableLength = 0, uStrLen;
	UCHAR	ucChecksum;
 	PUCHAR  pVirtualBaseAddress = NULL;
	ULONG 	i;
	PSMBIOSEntryPoint	pSmEp;
	do
	{
	    ConsolePrintf ("\n GetIPMIDeviceInformation: In");
		// Initialize the SMBIOS memory base address and range
		paSmBiosAddress = (PUCHAR)SMBIOS_MEM_RANGE_BASE;
		ulLength = SMBIOS_MAPPED_RANGE_LENGTH; //sundar change t0 0xFFFF

        //ntStatus = MapPhysicalToVirtualMem(pDeviceExtension, paSmBiosAddress, &ulLength, &pVirtualBaseAddress);
		//pVirtualBaseAddress = ioremap((ULONG)paSmBiosAddress, ulLength);
		if (MapPhysicalMemory((int)paSmBiosAddress, ulLength, (int *)&pVirtualBaseAddress) != ACCESN_OK)
		{
      		ConsolePrintf ( "\n No Virtual pointer to BIOS Memory.");
			return bStatus;
		}
	    ConsolePrintf ("Virtual Pointer: %x, Length: %x Scanning Signature...", pVirtualBaseAddress, ulLength);
		uStrLen = sizeof (SMBIOS_STRING) - 1;
		pSmEp = NULL;
        
		for(i=0; i<ulLength - uStrLen; i++)
		{
			// Look for "_SM_" signature in the BIOS that defines the start of
			// entry point structure.
			if( !memcmp((char *)(pVirtualBaseAddress + i), (char *)SMBIOS_STRING, uStrLen ))
			{
        		pSmEp = (PSMBIOSEntryPoint)(pVirtualBaseAddress+i);
		        ConsolePrintf ("SMBIOS Entry Point @ %x, offset: %x", pSmEp, i);
				break;
			}
		}
		if (!pSmEp)
    	{
      		ConsolePrintf ("Unable to Locate SMBIOS signature.");
			bStatus = FALSE;
			break;
		}
		
		// Verify checksum of the structure...
		for(i=0, ucChecksum=0x00; i<pSmEp->byLength; i++)
		{
            ucChecksum += (UCHAR) *((PUCHAR)pSmEp + i);
		}
		if (ucChecksum)
		{
            ConsolePrintf ("Invalid BIOS Entrypoint Checksum.");
			bStatus = FALSE;
			break;
		}
		// Check whether the version of the SMBIOS is compatible for
		// IPMI device information.
		if ((pSmEp->bySmBiosMajorVersion < SMBIOS_MAJOR_REVISION) || \
			(pSmEp->bySmBiosMinorVersion < SMBIOS_MINOR_REVISION))
		{
            ConsolePrintf (" SMBIOS Incompatible Version (%x.%x) detected.", \
            pSmEp->bySmBiosMajorVersion, pSmEp->bySmBiosMinorVersion);
			bStatus = FALSE;
			break;
		}

        ConsolePrintf ("SMBIOS Table Address - %x,Length:%x",pSmEp->ulTableAddress,pSmEp->wTableLength);
		// Initialize the table address and length and call scanforipmideviceinfo
        paSmTableAddress = (PUCHAR)pSmEp->ulTableAddress;
        ulTableLength = (ULONG) pSmEp->wTableLength;

        bStatus = ScanForIPMIDeviceInfo(pImpIf, paSmTableAddress, ulTableLength);
		if (!bStatus)
		{
			ConsolePrintf ( "Record Type 38 Could NOT be located.");
			break;
		}
	}
	while (FALSE);
	// Unmap virtual memory when everything is done...
	UnmapPhysicalMemory((int)pVirtualBaseAddress); // new function -san
    ConsolePrintf ( "GetIPMIDeviceInformation: Out. Status: %x", bStatus);
    return bStatus;
}

